//
// File:       <cutDems.C>
// Author:     <Remek Trzaska>
// e-mail:     <remek@npac.syr.edu>
// Created:    <06/03/97>
// Copyright Remek Trzaska 1997 NPAC, Syracuse University

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <math.h>

static const int DEM_LEVEL = 145;
static const int DEM_BOUNDARIES = 547;
static const int DEM_DATAOFFSET = 866;

int south, west;
int** data;

int parseDEM(fstream& dem)
{
   int type;
   float swe, swn, nwe, nwn, nee, nen, see, sen;

   dem.seekg(DEM_LEVEL, fstream::beg);
   dem >> type;
   dem.seekg(DEM_BOUNDARIES, fstream::beg);
   dem >> swe >> swn >> nwe >> nwn >> nee >> nen >> see >> sen;

   swe /= 3600;
   swn /= 3600;

   south = swn;
   west = -swe;

   return 1;
}

int readData(fstream& dem)
{
   dem.seekg(DEM_DATAOFFSET, fstream::beg);

   int curRow, curCol;
   int intDummy;
   float floatDummy;

   for(curRow=0; curRow<1201; curRow++)
   {
      dem >> intDummy;
      dem >> intDummy;
      if(intDummy!=curRow+1)
      {
	 cerr << "Wrong row number at position " << dem.tellg() << "!\n";
	 return 0;
      }
      dem >> intDummy;
      dem >> intDummy;
      dem >> floatDummy;
      dem >> floatDummy;
      dem >> floatDummy;
      dem >> floatDummy;
      dem >> floatDummy;
      for(curCol=0; curCol<1201; curCol++) 
      {
	 dem >> intDummy;
	 data[1201-curCol-1][curRow] = intDummy;
      }
      
   }
   return 1;
};

int saveChunk(int lat, int lon, int degSize)
{
   char buf[256];
   int reversedLat = 60/degSize-lat-1;
   
   float w = west - lon*degSize/60.0;
   float s = south + (reversedLat+1)*degSize/60.0;
   
   sprintf(buf, "%d:%d:%d,%d:%d:%d.dem",
	   (int)ffloor(w), (60 - lon*degSize)%60, 0,
	   (int)ffloor(s), ((reversedLat+1)*degSize)%60, 0);

   cerr << lon << ":" << lat << " becomes "
	<< buf << endl;
   
   int chunkSize = (int)(degSize/60.0*1201);
   
   ofstream aChunk(buf);
   sprintf(buf, "%s.pgm", buf);
   ofstream ppmChunk(buf);
   aChunk.put(0);
   aChunk.put(0);
   aChunk.put(chunkSize/256);
   aChunk.put(chunkSize%256);
   aChunk.put(0);
   aChunk.put(0);
   aChunk.put(chunkSize/256);
   aChunk.put(chunkSize%256);

   ppmChunk << "P5" << endl << chunkSize << " " << chunkSize << endl;
   ppmChunk << "255\n";

   for(int y=0; y<chunkSize; y++)
      for(int x=0; x<chunkSize; x++)
      {
	 aChunk.put(data[lat*chunkSize+(chunkSize-x-1)][lon*chunkSize+y]/256);
	 aChunk.put(data[lat*chunkSize+(chunkSize-x-1)][lon*chunkSize+y]%256);
	 ppmChunk.put(data[lat*chunkSize+y][lon*chunkSize+x]);
      }
   
   aChunk.close();
   ppmChunk.close();

   return 1;
}

int splitData(int degSize)
{
   for(int lat=0; lat < 60/degSize; lat++)
      for(int lon=0; lon < 60/degSize; lon++)
	 if(!saveChunk(lat, lon, degSize))
	    return 0;
   
   return 1;
};

void main(int argc, char* argv[])
{
   fstream dem(argv[1], ios::in);
   if(!parseDEM(dem))
   {
      cerr << "Cannot parse the file!\n";
      return;
   }

   int degSize = atoi(argv[2]);
   if(degSize!=15 && degSize!=30 && degSize!=5)
   {
      cerr << "Can only split into 5, 15 or 30 minute chunks!\n";
      dem.close();
      return;
   }

   data = new int *[1201];
   for(int i = 0; i<1201; i++)
      data[i] = new int[1201];
   
   if(!readData(dem))
   {
      cerr << "A problem while reading data!\n";
      dem.close();
      return;
   }
   
   if(!splitData(degSize))
   {
      cerr << "A problem while saving split chunks!\n";
      dem.close();
      return;
   }

   dem.close();
}

   
